/* -------------------------------------------------------------------------- */
/* Copyright 2002-2011, GridWay Project Leads (GridWay.org)                   */
/*                                                                            */
/* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
/* not use this file except in compliance with the License. You may obtain    */
/* a copy of the License at                                                   */
/*                                                                            */
/* http://www.apache.org/licenses/LICENSE-2.0                         	      */
/*                                                                            */
/* Unless required by applicable law or agreed to in writing, software        */
/* distributed under the License is distributed on an "AS IS" BASIS,          */
/* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   */
/* See the License for the specific language governing permissions and        */
/* limitations under the License.                                             */
/* -------------------------------------------------------------------------  */

package org.ggf.drmaa;

/**
  * The base class of the non-runtime DRMAAA exceptions.
  */

public class DrmaaException extends java.lang.Exception 
{
	 
	public DrmaaException() 
	{
		super();
	}
	
	public DrmaaException(java.lang.String message)
	{
		super(message);
	}

	public DrmaaException(Throwable throwable)
	{
		super(throwable);
	}

	public DrmaaException(String message, Throwable throwable)
	{
		super(message, throwable);
	}

}
